# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ConnectionBandwidth(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'qos_class': 'str',
        'capacity': 'str'
    }

    attribute_map = {
        'qos_class': 'qos_class',
        'capacity': 'capacity'
    }

    def __init__(self, qos_class='bestEffort', capacity='1024'):  # noqa: E501
        """ConnectionBandwidth - a model defined in Swagger"""  # noqa: E501
        self._qos_class = None
        self._capacity = None
        self.discriminator = None
        if qos_class is not None:
            self.qos_class = qos_class
        if capacity is not None:
            self.capacity = capacity

    @property
    def qos_class(self):
        """Gets the qos_class of this ConnectionBandwidth.  # noqa: E501

        Quality of Service (QoS) network priority.  # noqa: E501

        :return: The qos_class of this ConnectionBandwidth.  # noqa: E501
        :rtype: str
        """
        return self._qos_class

    @qos_class.setter
    def qos_class(self, qos_class):
        """Sets the qos_class of this ConnectionBandwidth.

        Quality of Service (QoS) network priority.  # noqa: E501

        :param qos_class: The qos_class of this ConnectionBandwidth.  # noqa: E501
        :type: str
        """
        allowed_values = ["guaranteedCapped", "softCapped", "anyAvailable", "bestEffort"]  # noqa: E501
        if qos_class not in allowed_values:
            raise ValueError(
                "Invalid value for `qos_class` ({0}), must be one of {1}"  # noqa: E501
                .format(qos_class, allowed_values)
            )

        self._qos_class = qos_class

    @property
    def capacity(self):
        """Gets the capacity of this ConnectionBandwidth.  # noqa: E501

        Bandwidth limit in Mbps.  # noqa: E501

        :return: The capacity of this ConnectionBandwidth.  # noqa: E501
        :rtype: str
        """
        return self._capacity

    @capacity.setter
    def capacity(self, capacity):
        """Sets the capacity of this ConnectionBandwidth.

        Bandwidth limit in Mbps.  # noqa: E501

        :param capacity: The capacity of this ConnectionBandwidth.  # noqa: E501
        :type: str
        """

        self._capacity = capacity

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ConnectionBandwidth, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConnectionBandwidth):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
